package lec21v4;

/*
 * Disclaimer.  This code was solely intended to support
 * the concepts covered in lecture 21.  It is not to be construded
 * as representative of good naming style, etc.
 * 
 * Implementation of exercise 5.1 & 5.2, push model, private version
 * of notifyObservers
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class FrameTester 
{
	public static void main(String[] args)
	{
		JFrame frame = new JFrame();
		frame.setLayout(new GridLayout(1,2));
		
		final Model lModel = new Model(3);
		FieldPanel lFields = new FieldPanel( lModel );
		BarPanel lBars = new BarPanel( lModel );
		lModel.addListener( lBars );
		lModel.addListener( lFields );
			
		frame.add( lFields );
		frame.add( lBars );
		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
	}
}

class FieldPanel extends JPanel implements ModelListener
{
	private JTextField[] aFields;
	private final static int FIELD_LENGTH = 3;
	private final static String DEFAULT_VALUE = "0";
	private Model aModel;
	
	public void modelChanged( int i, int v )
	{
		aFields[i].setText( new Integer( v ).toString() );
	}
	
	public FieldPanel( Model p )
	{
		aModel = p;
		aFields = new JTextField[ p.numValues() ];
		setLayout( new GridLayout( p.numValues(), 1 ));
		for( int i = 0; i < aFields.length; i++ )
		{
			aFields[i] = new JTextField( FIELD_LENGTH );
			aFields[i].setText(DEFAULT_VALUE);
			add( aFields[i] );
			aFields[i].addActionListener( new ActionListener() 
					{
						public void actionPerformed( ActionEvent e )
						{
							for( int i = 0; i < aFields.length; i++ )
							{
								if( e.getSource() == aFields[i] )
								{
									aModel.setValue( i, Integer.parseInt( aFields[i].getText()));
								}
							}
						}
					});
		}
	}
}

class BarPanel extends JPanel implements ModelListener
{
	private int[] aBars;
	private Model aModel;
	
	public BarPanel( Model pModel )
	{
		aModel = pModel;
		aBars = new int[pModel.numValues()];
		addMouseListener( new MouseAdapter()
			{
		public void mousePressed( MouseEvent arg0 )
		{
			super.mousePressed(arg0);
			for( int i = 0; i < aBars.length; i++ )
			{
				if( arg0.getX() > i*12 && arg0.getX() < (i*12 + 10))
				{
					aModel.setValue( i, getHeight() - arg0.getY() );
				}
			}
		}} );
	}
	
	public void paint( Graphics pGraphics )
	{
		super.paint( pGraphics );
		Graphics2D g2 = (Graphics2D) pGraphics;
		g2.setColor( Color.BLUE );
		for( int i = 0; i < aBars.length; i++ )
		{
			g2.fill3DRect(i*12,getHeight()-aBars[i],10,aBars[i], true);
		}
	}
	
	public void modelChanged( int i, int v )
	{
		aBars[i] = v;
		repaint();
	}
	
}

class Model
{
	private int[] aValues;
	private List<ModelListener> aListeners;
	
	public Model( int pNbValues )
	{
		aListeners = new LinkedList<ModelListener>();
		aValues = new int[ pNbValues ];
		for( int i = 0; i < pNbValues; i++ )
		{
			aValues[i] = 0;
		}
	}
	
	public int numValues()
	{
		return aValues.length;
	}
	
	public int getValue( int p )
	{
		return aValues[p];
	}
	
	public void setValue( int p, int v )
	{
		aValues[p] = v;
		notifyListeners( p, v );
	}
	
	public void addListener( ModelListener pListener )
	{
		aListeners.add( pListener );
	}
	
	public void removeListener( ModelListener pListener )
	{
		aListeners.remove( pListener );
	}
	
	private void notifyListeners( int i, int v )
	{
		for( ModelListener l : aListeners )
		{
			l.modelChanged( i, v );
		}
			
	}
}

interface ModelListener
{
	void modelChanged( int i, int v );
}
